@@ -142,38 +142,7 @@ static void
 xsltFreeStylePreComp(xsltStylePreCompPtr comp) {
     if (comp == NULL)
 	return;
-    if (comp->stype != NULL)
-	xmlFree(comp->stype);
-    if (comp->order != NULL)
-	xmlFree(comp->order);
-    if (comp->use != NULL)
-	xmlFree(comp->use);
-    if (comp->name != NULL)
-	xmlFree(comp->name);
-    if (comp->ns != NULL)
-	xmlFree(comp->ns);
-    if (comp->mode != NULL)
-	xmlFree(comp->mode);
-    if (comp->modeURI != NULL)
-	xmlFree(comp->modeURI);
-    if (comp->test != NULL)
-	xmlFree(comp->test);
-    if (comp->select != NULL)
-	xmlFree(comp->select);
 
-    if (comp->filename != NULL)
-	xmlFree(comp->filename);
-
-    if (comp->numdata.level != NULL)
-	xmlFree(comp->numdata.level);
-    if (comp->numdata.count != NULL)
-	xmlFree(comp->numdata.count);
-    if (comp->numdata.from != NULL)
-	xmlFree(comp->numdata.from);
-    if (comp->numdata.value != NULL)
-	xmlFree(comp->numdata.value);
-    if (comp->numdata.format != NULL)
-	xmlFree(comp->numdata.format);
     if (comp->comp != NULL)
 	xmlXPathFreeCompExpr(comp->comp);
     if (comp->nsList != NULL)
@@ -203,8 +172,7 @@ xsltElemPreCompPtr
 xsltDocumentComp(xsltStylesheetPtr style, xmlNodePtr inst,
 		 xsltTransformFunction function ATTRIBUTE_UNUSED) {
     xsltStylePreCompPtr comp;
-    xmlChar *filename = NULL;
-    xmlChar *URL = NULL;
+    const xmlChar *filename = NULL;
 
     comp = xsltNewStylePreComp(style, XSLT_FUNC_DOCUMENT);
     if (comp == NULL)
@@ -250,12 +218,7 @@ xsltDocumentComp(xsltStylesheetPtr style, xmlNodePtr inst,
     if (!comp->has_filename) {
 	goto error;
     }
-
-    if (filename != NULL) {
-	comp->filename = filename;
-    } else {
-	comp->filename = NULL;
-    }
+    comp->filename = filename;
 
 error:
     return ((xsltElemPreCompPtr) comp);
@@ -336,16 +299,16 @@ xsltSortComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 				 (const xmlChar *)"lang",
 				 XSLT_NAMESPACE, &comp->has_lang);
 
-    comp->select = xsltGetNsProp(inst,(const xmlChar *)"select", XSLT_NAMESPACE);
+    comp->select = xsltGetCNsProp(style, inst,(const xmlChar *)"select", XSLT_NAMESPACE);
     if (comp->select == NULL) {
 	/*
 	 * The default value of the select attribute is ., which will
 	 * cause the string-value of the current node to be used as
 	 * the sort key.
 	 */
-	comp->select = xmlStrdup((const xmlChar *)".");
+	comp->select = xmlDictLookup(style->dict, BAD_CAST ".", 1);
     }
-    comp->comp = xmlXPathCompile(comp->select);
+    comp->comp = xsltXPathCompile(style, comp->select);
     if (comp->comp == NULL) {
 	xsltTransformError(NULL, style, inst,
 	     "xsltSortComp: could not compile select expression '%s'\n",
@@ -380,7 +343,7 @@ xsltCopyComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     comp->inst = inst;
 
 
-    comp->use = xsltGetNsProp(inst, (const xmlChar *)"use-attribute-sets",
+    comp->use = xsltGetCNsProp(style, inst, (const xmlChar *)"use-attribute-sets",
 				    XSLT_NAMESPACE);
     if (comp->use == NULL)
 	comp->has_use = 0;
@@ -390,7 +353,7 @@ xsltCopyComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltTextComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt text node
  *
  * Process the xslt text node on the source node
@@ -398,7 +361,7 @@ xsltCopyComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 static void
 xsltTextComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     xsltStylePreCompPtr comp;
-    xmlChar *prop;
+    const xmlChar *prop;
 
     if ((style == NULL) || (inst == NULL))
 	return;
@@ -409,7 +372,7 @@ xsltTextComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     comp->inst = inst;
     comp->noescape = 0;
 
-    prop = xsltGetNsProp(inst,
+    prop = xsltGetCNsProp(style, inst,
 	    (const xmlChar *)"disable-output-escaping",
 			XSLT_NAMESPACE);
     if (prop != NULL) {
@@ -421,13 +384,12 @@ xsltTextComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 "xsl:text: disable-output-escaping allows only yes or no\n");
 	    if (style != NULL) style->warnings++;
 	}
-	xmlFree(prop);
     }
 }
 
 /**
  * xsltElementComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt element node
  *
  * Process the xslt element node on the source node
@@ -444,12 +406,16 @@ xsltElementComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     inst->_private = comp;
     comp->inst = inst;
 
-    /*
-     * TODO: more computation can be done there, especially namespace lookup
-     */
     comp->name = xsltEvalStaticAttrValueTemplate(style, inst,
 				 (const xmlChar *)"name",
 				 XSLT_NAMESPACE, &comp->has_name);
+    if (comp->name != NULL) {
+	if (xmlValidateQName(comp->name, 0)) {
+	    xsltTransformError(NULL, style, inst,
+		    "xsl:element : invalid name\n");
+	    if (style != NULL) style->errors++;
+	}
+    }
     comp->ns = xsltEvalStaticAttrValueTemplate(style, inst,
 			 (const xmlChar *)"namespace",
 			 XSLT_NAMESPACE, &comp->has_ns);
@@ -458,7 +424,7 @@ xsltElementComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 	defaultNs = xmlSearchNs(inst->doc, inst, NULL);
 	if (defaultNs != NULL) {
-	    comp->ns = xmlStrdup(defaultNs->href);
+	    comp->ns = xmlDictLookup(style->dict, defaultNs->href, -1);
 	    comp->has_ns = 1;
 	}
     }
@@ -469,7 +435,7 @@ xsltElementComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltAttributeComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt attribute node
  *
  * Process the xslt attribute node on the source node
@@ -492,6 +458,13 @@ xsltAttributeComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     comp->name = xsltEvalStaticAttrValueTemplate(style, inst,
 				 (const xmlChar *)"name",
 				 XSLT_NAMESPACE, &comp->has_name);
+    if (comp->name != NULL) {
+	if (xmlValidateQName(comp->name, 0)) {
+	    xsltTransformError(NULL, style, inst,
+			    "xsl:attribute : invalid QName\n");
+	    if (style != NULL) style->errors++;
+	}
+    }
     comp->ns = xsltEvalStaticAttrValueTemplate(style, inst,
 			 (const xmlChar *)"namespace",
 			 XSLT_NAMESPACE, &comp->has_ns);
@@ -500,7 +473,7 @@ xsltAttributeComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltCommentComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt comment node
  *
  * Process the xslt comment node on the source node
@@ -520,7 +493,7 @@ xsltCommentComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltProcessingInstructionComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt processing-instruction node
  *
  * Process the xslt processing-instruction node on the source node
@@ -544,7 +517,7 @@ xsltProcessingInstructionComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltCopyOfComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt copy-of node
  *
  * Process the xslt copy-of node on the source node
@@ -561,7 +534,7 @@ xsltCopyOfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     inst->_private = comp;
     comp->inst = inst;
 
-    comp->select = xsltGetNsProp(inst, (const xmlChar *)"select",
+    comp->select = xsltGetCNsProp(style, inst, (const xmlChar *)"select",
 	                        XSLT_NAMESPACE);
     if (comp->select == NULL) {
 	xsltTransformError(NULL, style, inst,
@@ -569,7 +542,7 @@ xsltCopyOfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	if (style != NULL) style->errors++;
 	return;
     }
-    comp->comp = xmlXPathCompile(comp->select);
+    comp->comp = xsltXPathCompile(style, comp->select);
     if (comp->comp == NULL) {
 	xsltTransformError(NULL, style, inst,
 	     "xsl:copy-of : could not compile select expression '%s'\n",
@@ -580,7 +553,7 @@ xsltCopyOfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltValueOfComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt value-of node
  *
  * Process the xslt value-of node on the source node
@@ -588,7 +561,7 @@ xsltCopyOfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 static void
 xsltValueOfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     xsltStylePreCompPtr comp;
-    xmlChar *prop;
+    const xmlChar *prop;
 
     if ((style == NULL) || (inst == NULL))
 	return;
@@ -598,7 +571,7 @@ xsltValueOfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     inst->_private = comp;
     comp->inst = inst;
 
-    prop = xsltGetNsProp(inst,
+    prop = xsltGetCNsProp(style, inst,
 	    (const xmlChar *)"disable-output-escaping",
 			XSLT_NAMESPACE);
     if (prop != NULL) {
@@ -610,9 +583,8 @@ xsltValueOfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 "xsl:value-of : disable-output-escaping allows only yes or no\n");
 	    if (style != NULL) style->warnings++;
 	}
-	xmlFree(prop);
     }
-    comp->select = xsltGetNsProp(inst, (const xmlChar *)"select",
+    comp->select = xsltGetCNsProp(style, inst, (const xmlChar *)"select",
 	                        XSLT_NAMESPACE);
     if (comp->select == NULL) {
 	xsltTransformError(NULL, style, inst,
@@ -620,7 +592,7 @@ xsltValueOfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	if (style != NULL) style->errors++;
 	return;
     }
-    comp->comp = xmlXPathCompile(comp->select);
+    comp->comp = xsltXPathCompile(style, comp->select);
     if (comp->comp == NULL) {
 	xsltTransformError(NULL, style, inst,
 	     "xsl:value-of : could not compile select expression '%s'\n",
@@ -631,7 +603,7 @@ xsltValueOfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltWithParamComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt with-param node
  *
  * Process the xslt with-param node on the source node
@@ -639,7 +611,7 @@ xsltValueOfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 static void
 xsltWithParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     xsltStylePreCompPtr comp;
-    xmlChar *prop;
+    const xmlChar *prop;
 
     if ((style == NULL) || (inst == NULL))
 	return;
@@ -652,7 +624,7 @@ xsltWithParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     /*
      * The full namespace resolution can be done statically
      */
-    prop = xsltGetNsProp(inst, (const xmlChar *)"name", XSLT_NAMESPACE);
+    prop = xsltGetCNsProp(style, inst, (const xmlChar *)"name", XSLT_NAMESPACE);
     if (prop == NULL) {
 	xsltTransformError(NULL, style, inst,
 	     "xsl:with-param : name is missing\n");
@@ -660,7 +632,7 @@ xsltWithParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     } else {
         const xmlChar *URI;
 
-	URI = xsltGetQNameURI(inst, &prop);
+	URI = xsltGetQNameURI(inst, (xmlChar **)&prop);
 	if (prop == NULL) {
 	    if (style != NULL) style->errors++;
 	} else {
@@ -675,10 +647,10 @@ xsltWithParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	}
     }
 
-    comp->select = xsltGetNsProp(inst, (const xmlChar *)"select",
+    comp->select = xsltGetCNsProp(style, inst, (const xmlChar *)"select",
 	                        XSLT_NAMESPACE);
     if (comp->select != NULL) {
-	comp->comp = xmlXPathCompile(comp->select);
+	comp->comp = xsltXPathCompile(style, comp->select);
 	if (comp->comp == NULL) {
 	    xsltTransformError(NULL, style, inst,
 		 "xsl:param : could not compile select expression '%s'\n",
@@ -695,7 +667,7 @@ xsltWithParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltNumberComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @cur:   the xslt number node
  *
  * Process the xslt number node on the source node
@@ -703,7 +675,7 @@ xsltWithParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 static void
 xsltNumberComp(xsltStylesheetPtr style, xmlNodePtr cur) {
     xsltStylePreCompPtr comp;
-    xmlChar *prop;
+    const xmlChar *prop;
 
     if ((style == NULL) || (cur == NULL))
 	return;
@@ -717,24 +689,24 @@ xsltNumberComp(xsltStylesheetPtr style, xmlNodePtr cur) {
 
     comp->numdata.doc = cur->doc;
     comp->numdata.node = cur;
-    comp->numdata.value = xsltGetNsProp(cur, (const xmlChar *)"value",
+    comp->numdata.value = xsltGetCNsProp(style, cur, (const xmlChar *)"value",
 	                                XSLT_NAMESPACE);
     
     prop = xsltEvalStaticAttrValueTemplate(style, cur,
 			 (const xmlChar *)"format",
 			 XSLT_NAMESPACE, &comp->numdata.has_format);
     if (comp->numdata.has_format == 0) {
-	comp->numdata.format = xmlStrdup(BAD_CAST(""));
+	comp->numdata.format = xmlDictLookup(style->dict, BAD_CAST "" , 0);
     } else {
 	comp->numdata.format = prop;
     }
 
-    comp->numdata.count = xsltGetNsProp(cur, (const xmlChar *)"count",
+    comp->numdata.count = xsltGetCNsProp(style, cur, (const xmlChar *)"count",
 					XSLT_NAMESPACE);
-    comp->numdata.from = xsltGetNsProp(cur, (const xmlChar *)"from",
+    comp->numdata.from = xsltGetCNsProp(style, cur, (const xmlChar *)"from",
 					XSLT_NAMESPACE);
     
-    prop = xsltGetNsProp(cur, (const xmlChar *)"level", XSLT_NAMESPACE);
+    prop = xsltGetCNsProp(style, cur, (const xmlChar *)"level", XSLT_NAMESPACE);
     if (prop != NULL) {
 	if (xmlStrEqual(prop, BAD_CAST("single")) ||
 	    xmlStrEqual(prop, BAD_CAST("multiple")) ||
@@ -744,17 +716,17 @@ xsltNumberComp(xsltStylesheetPtr style, xmlNodePtr cur) {
 	    xsltTransformError(NULL, style, cur,
 			 "xsl:number : invalid value %s for level\n", prop);
 	    if (style != NULL) style->warnings++;
-	    xmlFree(prop);
+	    xmlFree((void *)(prop));
 	}
     }
     
-    prop = xsltGetNsProp(cur, (const xmlChar *)"lang", XSLT_NAMESPACE);
+    prop = xsltGetCNsProp(style, cur, (const xmlChar *)"lang", XSLT_NAMESPACE);
     if (prop != NULL) {
 	XSLT_TODO; /* xsl:number lang attribute */
-	xmlFree(prop);
+	xmlFree((void *)prop);
     }
     
-    prop = xsltGetNsProp(cur, (const xmlChar *)"letter-value", XSLT_NAMESPACE);
+    prop = xsltGetCNsProp(style, cur, (const xmlChar *)"letter-value", XSLT_NAMESPACE);
     if (prop != NULL) {
 	if (xmlStrEqual(prop, BAD_CAST("alphabetic"))) {
 	    xsltTransformError(NULL, style, cur,
@@ -771,21 +743,19 @@ xsltNumberComp(xsltStylesheetPtr style, xmlNodePtr cur) {
 		     "xsl:number : invalid value %s for letter-value\n", prop);
 	    if (style != NULL) style->warnings++;
 	}
-	xmlFree(prop);
     }
     
-    prop = xsltGetNsProp(cur, (const xmlChar *)"grouping-separator",
+    prop = xsltGetCNsProp(style, cur, (const xmlChar *)"grouping-separator",
 	                XSLT_NAMESPACE);
     if (prop != NULL) {
+        comp->numdata.groupingCharacterLen = xmlStrlen(prop);
 	comp->numdata.groupingCharacter =
 	    xsltGetUTF8Char(prop, &(comp->numdata.groupingCharacterLen));
-	xmlFree(prop);
     }
     
-    prop = xsltGetNsProp(cur, (const xmlChar *)"grouping-size", XSLT_NAMESPACE);
+    prop = xsltGetCNsProp(style, cur, (const xmlChar *)"grouping-size", XSLT_NAMESPACE);
     if (prop != NULL) {
 	sscanf((char *)prop, "%d", &comp->numdata.digitsPerGroup);
-	xmlFree(prop);
     } else {
 	comp->numdata.groupingCharacter = 0;
     }
@@ -793,7 +763,8 @@ xsltNumberComp(xsltStylesheetPtr style, xmlNodePtr cur) {
     /* Set default values */
     if (comp->numdata.value == NULL) {
 	if (comp->numdata.level == NULL) {
-	    comp->numdata.level = xmlStrdup(BAD_CAST("single"));
+	    comp->numdata.level = xmlDictLookup(style->dict,
+	                                        BAD_CAST"single", 6);
 	}
     }
     
@@ -801,7 +772,7 @@ xsltNumberComp(xsltStylesheetPtr style, xmlNodePtr cur) {
 
 /**
  * xsltApplyImportsComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt apply-imports node
  *
  * Process the xslt apply-imports node on the source node
@@ -821,7 +792,7 @@ xsltApplyImportsComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltCallTemplateComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt call-template node
  *
  * Process the xslt call-template node on the source node
@@ -829,7 +800,7 @@ xsltApplyImportsComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 static void
 xsltCallTemplateComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     xsltStylePreCompPtr comp;
-    xmlChar *prop;
+    const xmlChar *prop;
 
     if ((style == NULL) || (inst == NULL))
 	return;
@@ -842,7 +813,7 @@ xsltCallTemplateComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     /*
      * The full template resolution can be done statically
      */
-    prop = xsltGetNsProp(inst, (const xmlChar *)"name", XSLT_NAMESPACE);
+    prop = xsltGetCNsProp(style, inst, (const xmlChar *)"name", XSLT_NAMESPACE);
     if (prop == NULL) {
 	xsltTransformError(NULL, style, inst,
 	     "xsl:call-template : name is missing\n");
@@ -850,7 +821,7 @@ xsltCallTemplateComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     } else {
         const xmlChar *URI;
 
-	URI = xsltGetQNameURI(inst, &prop);
+	URI = xsltGetQNameURI(inst, (xmlChar **)&prop);
 	if (prop == NULL) {
 	    if (style != NULL) style->errors++;
 	} else {
@@ -869,7 +840,7 @@ xsltCallTemplateComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltApplyTemplatesComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the apply-templates node
  *
  * Process the apply-templates node on the source node
@@ -877,7 +848,7 @@ xsltCallTemplateComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 static void
 xsltApplyTemplatesComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     xsltStylePreCompPtr comp;
-    xmlChar *prop;
+    const xmlChar *prop;
 
     if ((style == NULL) || (inst == NULL))
 	return;
@@ -890,11 +861,11 @@ xsltApplyTemplatesComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     /*
      * Get mode if any
      */
-    prop = xsltGetNsProp(inst, (const xmlChar *)"mode", XSLT_NAMESPACE);
+    prop = xsltGetCNsProp(style, inst, (const xmlChar *)"mode", XSLT_NAMESPACE);
     if (prop != NULL) {
         const xmlChar *URI;
 
-	URI = xsltGetQNameURI(inst, &prop);
+	URI = xsltGetQNameURI(inst, (xmlChar **)&prop);
 	if (prop == NULL) {
 	    if (style != NULL) style->errors++;
 	} else {
@@ -906,10 +877,10 @@ xsltApplyTemplatesComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	    }
 	}
     }
-    comp->select = xsltGetNsProp(inst, (const xmlChar *)"select",
+    comp->select = xsltGetCNsProp(style, inst, (const xmlChar *)"select",
 	                        XSLT_NAMESPACE);
     if (comp->select != NULL) {
-	comp->comp = xmlXPathCompile(comp->select);
+	comp->comp = xsltXPathCompile(style, comp->select);
 	if (comp->comp == NULL) {
 	    xsltTransformError(NULL, style, inst,
      "xsl:apply-templates : could not compile select expression '%s'\n",
@@ -923,7 +894,7 @@ xsltApplyTemplatesComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltChooseComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt choose node
  *
  * Process the xslt choose node on the source node
@@ -943,7 +914,7 @@ xsltChooseComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltIfComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt if node
  *
  * Process the xslt if node on the source node
@@ -960,14 +931,14 @@ xsltIfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     inst->_private = comp;
     comp->inst = inst;
 
-    comp->test = xsltGetNsProp(inst, (const xmlChar *)"test", XSLT_NAMESPACE);
+    comp->test = xsltGetCNsProp(style, inst, (const xmlChar *)"test", XSLT_NAMESPACE);
     if (comp->test == NULL) {
 	xsltTransformError(NULL, style, inst,
 	     "xsl:if : test is not defined\n");
 	if (style != NULL) style->errors++;
 	return;
     }
-    comp->comp = xmlXPathCompile(comp->test);
+    comp->comp = xsltXPathCompile(style, comp->test);
     if (comp->comp == NULL) {
 	xsltTransformError(NULL, style, inst,
 	     "xsl:if : could not compile test expression '%s'\n",
@@ -978,7 +949,7 @@ xsltIfComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltWhenComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt if node
  *
  * Process the xslt if node on the source node
@@ -995,14 +966,14 @@ xsltWhenComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     inst->_private = comp;
     comp->inst = inst;
 
-    comp->test = xsltGetNsProp(inst, (const xmlChar *)"test", XSLT_NAMESPACE);
+    comp->test = xsltGetCNsProp(style, inst, (const xmlChar *)"test", XSLT_NAMESPACE);
     if (comp->test == NULL) {
 	xsltTransformError(NULL, style, inst,
 	     "xsl:when : test is not defined\n");
 	if (style != NULL) style->errors++;
 	return;
     }
-    comp->comp = xmlXPathCompile(comp->test);
+    comp->comp = xsltXPathCompile(style, comp->test);
     if (comp->comp == NULL) {
 	xsltTransformError(NULL, style, inst,
 	     "xsl:when : could not compile test expression '%s'\n",
@@ -1013,7 +984,7 @@ xsltWhenComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltForEachComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt for-each node
  *
  * Process the xslt for-each node on the source node
@@ -1030,14 +1001,14 @@ xsltForEachComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     inst->_private = comp;
     comp->inst = inst;
 
-    comp->select = xsltGetNsProp(inst, (const xmlChar *)"select",
+    comp->select = xsltGetCNsProp(style, inst, (const xmlChar *)"select",
 	                        XSLT_NAMESPACE);
     if (comp->select == NULL) {
 	xsltTransformError(NULL, style, inst,
 		"xsl:for-each : select is missing\n");
 	if (style != NULL) style->errors++;
     } else {
-	comp->comp = xmlXPathCompile(comp->select);
+	comp->comp = xsltXPathCompile(style, comp->select);
 	if (comp->comp == NULL) {
 	    xsltTransformError(NULL, style, inst,
      "xsl:for-each : could not compile select expression '%s'\n",
@@ -1050,7 +1021,7 @@ xsltForEachComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltVariableComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt variable node
  *
  * Process the xslt variable node on the source node
@@ -1058,7 +1029,7 @@ xsltForEachComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 static void
 xsltVariableComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     xsltStylePreCompPtr comp;
-    xmlChar *prop;
+    const xmlChar *prop;
 
     if ((style == NULL) || (inst == NULL))
 	return;
@@ -1071,7 +1042,7 @@ xsltVariableComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     /*
      * The full template resolution can be done statically
      */
-    prop = xsltGetNsProp(inst, (const xmlChar *)"name", XSLT_NAMESPACE);
+    prop = xsltGetCNsProp(style, inst, (const xmlChar *)"name", XSLT_NAMESPACE);
     if (prop == NULL) {
 	xsltTransformError(NULL, style, inst,
 	     "xsl:variable : name is missing\n");
@@ -1079,7 +1050,7 @@ xsltVariableComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     } else {
         const xmlChar *URI;
 
-	URI = xsltGetQNameURI(inst, &prop);
+	URI = xsltGetQNameURI(inst, (xmlChar **)&prop);
 	if (prop == NULL) {
 	    if (style != NULL) style->errors++;
 	} else {
@@ -1094,10 +1065,10 @@ xsltVariableComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	}
     }
 
-    comp->select = xsltGetNsProp(inst, (const xmlChar *)"select",
+    comp->select = xsltGetCNsProp(style, inst, (const xmlChar *)"select",
 	                        XSLT_NAMESPACE);
     if (comp->select != NULL) {
-	comp->comp = xmlXPathCompile(comp->select);
+	comp->comp = xsltXPathCompile(style, comp->select);
 	if (comp->comp == NULL) {
 	    xsltTransformError(NULL, style, inst,
 		 "xsl:variable : could not compile select expression '%s'\n",
@@ -1114,7 +1085,7 @@ xsltVariableComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 
 /**
  * xsltParamComp:
- * @style:  a XSLT process context
+ * @style: an XSLT compiled stylesheet
  * @inst:  the xslt param node
  *
  * Process the xslt param node on the source node
@@ -1122,7 +1093,7 @@ xsltVariableComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 static void
 xsltParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     xsltStylePreCompPtr comp;
-    xmlChar *prop;
+    const xmlChar *prop;
 
     if ((style == NULL) || (inst == NULL))
 	return;
@@ -1135,7 +1106,7 @@ xsltParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     /*
      * The full template resolution can be done statically
      */
-    prop = xsltGetNsProp(inst, (const xmlChar *)"name", XSLT_NAMESPACE);
+    prop = xsltGetCNsProp(style, inst, (const xmlChar *)"name", XSLT_NAMESPACE);
     if (prop == NULL) {
 	xsltTransformError(NULL, style, inst,
 	     "xsl:param : name is missing\n");
@@ -1143,7 +1114,7 @@ xsltParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     } else {
         const xmlChar *URI;
 
-	URI = xsltGetQNameURI(inst, &prop);
+	URI = xsltGetQNameURI(inst, (xmlChar **)&prop);
 	if (prop == NULL) {
 	    if (style != NULL) style->errors++;
 	} else {
@@ -1158,10 +1129,10 @@ xsltParamComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	}
     }
 
-    comp->select = xsltGetNsProp(inst, (const xmlChar *)"select",
+    comp->select = xsltGetCNsProp(style, inst, (const xmlChar *)"select",
 	                        XSLT_NAMESPACE);
     if (comp->select != NULL) {
-	comp->comp = xmlXPathCompile(comp->select);
+	comp->comp = xsltXPathCompile(style, comp->select);
 	if (comp->comp == NULL) {
 	    xsltTransformError(NULL, style, inst,
 		 "xsl:param : could not compile select expression '%s'\n",
